import pygame
from pygame.locals import *
import numpy as np
import random

class Level(pygame.sprite.Group):    
    
    blockImage = pygame.image.load('block.png') 
    streetImage = pygame.image.load('street.png') 
        
    def __init__(self, centerLoc):
        pygame.sprite.Group.__init__(self)         
        self.interactiveObjects = pygame.sprite.Group()
        self.setupLevel(centerLoc)
    
    def setupLevel(self, centerLoc): 
        self.levelSprites = []

        size = 3
        self.size = 128*size*6
        
        for x_index in range(size):
            for y_index in range(size):
                x = x_index*5
                y = y_index*5
                self.addBlock((x,y)) 
                self.addStreetCol((x-1,y-1),6)
                self.addStreetCol((x+4,y-1),6)
                self.addStreetRow((x,y-1), 4)
                self.addStreetRow((x,y+4), 4)
        
        '''self.addBlock((5,0))
        self.addBlock((-3,5))
        self.addBlock((2,5))
        self.addBlock((7,5))
        
        self.addStreetRow((-3,4),12)
        self.addStreetCol((4,0),4) 
        self.addStreetCol((1,5),4) 
        self.addStreetCol((6,5),4)'''
        
        #set the initial position
        self.pan(np.array(centerLoc))
        
    def addBlock(self, loc):
        temp = pygame.sprite.Sprite(self)
        temp.image = self.blockImage
        temp.rect = self.blockImage.get_rect()
        pixelLoc = (loc[0]*128, loc[1]*128)
        temp.rect.topleft = pixelLoc
        self.add(temp)
        
        #add bouncy walls
        wallThickness = 50
        rhouse = pygame.Rect(temp.rect.left + 207, temp.rect.top+117, 139, 220)
        '''r = pygame.Rect(rhouse.left+wallThickness, rhouse.top, \
            rhouse.width - 2*wallThickness, wallThickness)
        BouncyWall((self.interactiveObjects, self), True, r)
        r = pygame.Rect(rhouse.left, rhouse.bottom, rhouse.width - 2*wallThickness, wallThickness)
        BouncyWall((self.interactiveObjects, self), True, r)
        r = pygame.Rect(rhouse.left, rhouse.top, wallThickness, rhouse.height)
        BouncyWall((self.interactiveObjects, self), False, r)
        r = pygame.Rect(rhouse.right-wallThickness, rhouse.top, wallThickness, rhouse.height)
        BouncyWall((self.interactiveObjects, self), False, r)'''
        
        #add bouncy circles for corners
        BouncyCircle((self.interactiveObjects, self), \
            np.add(rhouse.topleft, (wallThickness, wallThickness)), wallThickness)
        BouncyCircle((self.interactiveObjects, self), \
            np.add(rhouse.topright, (-wallThickness, wallThickness)), wallThickness)
        BouncyCircle((self.interactiveObjects, self), \
            np.add(rhouse.bottomleft, (wallThickness, 0)), wallThickness)
        BouncyCircle((self.interactiveObjects, self), \
            np.add(rhouse.bottomright, (-wallThickness,0)), wallThickness)
        BouncyCircle((self.interactiveObjects, self), \
            np.add(rhouse.bottomright, (-wallThickness,0)), wallThickness)
        BouncyCircle((self.interactiveObjects, self), \
            np.add(rhouse.midbottom, (0,-wallThickness)), wallThickness)
        BouncyCircle((self.interactiveObjects, self), \
            np.add(rhouse.midtop, (0,wallThickness)), wallThickness)
        BouncyCircle((self.interactiveObjects, self), \
            np.add(rhouse.midleft, (wallThickness, wallThickness)), wallThickness)
        BouncyCircle((self.interactiveObjects, self), \
            np.add(rhouse.midright, (-wallThickness, wallThickness)), wallThickness)
        BouncyCircle((self.interactiveObjects, self), \
            np.add(rhouse.midleft, (wallThickness, 0)), wallThickness)
        BouncyCircle((self.interactiveObjects, self), \
            np.add(rhouse.midright, (-wallThickness, 0)), wallThickness)
        
    def addStreetCol(self, loc, length):
        for i in range(length):
            temp = pygame.sprite.Sprite(self)
            temp.image = self.streetImage
            temp.rect = self.streetImage.get_rect()
            pixelLoc = (loc[0]*128, (loc[1]+i)*128)
            temp.rect.topleft = pixelLoc
            self.add(temp)   
            
    def addStreetRow(self, loc, length):
        for i in range(length):
            temp = pygame.sprite.Sprite(self)
            temp.image = self.streetImage
            temp.image = pygame.transform.rotate(temp.image, 90)
            temp.rect = self.streetImage.get_rect()
            pixelLoc = ((loc[0]+i)*128, loc[1]*128)
            temp.rect.topleft = pixelLoc
            self.add(temp)   
    
    def pan(self, pan):
        for s in self:
            s.rect.topleft+= pan

class BouncyWall(pygame.sprite.Sprite):
    def __init__(self, groups, isHorizontal, rect):
        pygame.sprite.Sprite.__init__(self, groups)
        self.isHorizontal = isHorizontal
        self.rect = rect
        self.image = pygame.Surface((self.rect.width, self.rect.height))
        #self.image.set_colorkey(pygame.Color(0,0,0))
    
    def bounce(self, s):
        if self.isHorizontal:
            #s.vbar[1] *= -1
            if s.rect.bottom > self.rect.top and s.rect.bottom < self.rect.bottom:
                s.xbar[1] -= s.rect.bottom - self.rect.top + 2
            else:
                s.xbar[1] -= s.rect.top - self.rect.bottom + 2
        else:
            #s.vbar[0] *= -1
            if s.rect.right > self.rect.left and s.rect.right < self.rect.right:
                s.xbar[0]  -= s.rect.right - self.rect.left + 2
            else:
                s.xbar[1] += self.rect.right - s.rect.left + 2
        s.rect.center = s.xbar

class BouncyCircle(pygame.sprite.Sprite):
    def __init__(self, groups, center, radius):
        pygame.sprite.Sprite.__init__(self, groups)
        self.image = pygame.Surface((radius*2, radius*2))
        #pygame.draw.circle(self.image, pygame.Color(255,255,255), (radius, radius), radius)
        self.rect = self.image.get_rect()
        self.rect.center = center
        self.radius = radius
        self.image.set_colorkey(pygame.Color(0,0,0))
        
    def bounce(self, s):
        rbar = np.subtract(s.rect.center, self.rect.center)
        r = np.linalg.norm(rbar)
        if r == 0:
            return
        rhat = rbar/r
        rdesired = self.radius + s.radius+2
        dxbar = rhat*(rdesired - r)
        s.xbar += dxbar
        s.rect.center = s.xbar
        '''
        v = np.linalg.norm(s.vbar)
        if v == 0:
            return
        vbar_radial = s.vbar*np.dot(s.vbar, rhat)/v
        s.vbar -= 2*vbar_radial
        print s.xbar, s.vbar'''