import pygame
import sys
from pygame.locals import *
import swarm_objects
import random
import numpy as np
import gui
import level

pygame.init()
random.seed()

windowSurface = pygame.display.set_mode((1000,800))
pygame.display.set_caption( 'Ape Bit!' )

clock = pygame.time.Clock()
fps_target = 25.

allSprites = pygame.sprite.Group()
zombieGroup = pygame.sprite.Group()
lampGroup = pygame.sprite.Group()

swarm_objects.Lamp((allSprites, lampGroup), (100,100), False)
swarm_objects.Lamp((allSprites, lampGroup), (700,700), True)
survivor = swarm_objects.Survivor(allSprites, (400,400))
gui1 = gui.Gui(clock)
level1 = level.Level((0,0))

level1.originLocation = np.array(survivor.xbar)

def panCameraTo(x, y):
    panTo = np.array((x, y)) - (windowSurface.get_width()/2, windowSurface.get_height()/2)
    for s in allSprites:
        s.xbar -= panTo
        s.rect.center -= panTo
    level1.pan(-panTo)
    level1.originLocation -= panTo

def draw():
    windowSurface.fill(pygame.Color(0,255,0))
    camera_x, camera_y = survivor.xbar
    panCameraTo(camera_x, camera_y)
    level1.draw(windowSurface)
    level1.interactiveObjects.draw(windowSurface)
    allSprites.draw(windowSurface)
    gui1.draw(windowSurface)
    pygame.display.update()

randZs = pygame.sprite.Group()

while True:
    #if gui1.gotBit():
    #survivor.showOutOfLife()
    #handle keyboard input

    #randomly add zombies
    if random.randint(0,1000) > 990:
        spawnLoc = None
        while spawnLoc == None:
            for s in level1:
                if s.image == level.Level.streetImage:
                    r = np.linalg.norm(survivor.xbar - s.rect.center)
                    if r > 400 and random.randint(0, len(level1)):
                        spawnLoc = s.rect.center
        swarm_objects.Zombie((zombieGroup, allSprites), \
            spawnLoc)
    
    keystate = pygame.key.get_pressed()
    survivor_accel = np.zeros(2)
    if keystate[K_UP]:
        survivor_accel += (0, -1)
    if keystate[K_DOWN]:
        survivor_accel += (0, 1)
    if keystate[K_RIGHT]:
        survivor_accel += (1, 0)
    if keystate[K_LEFT]:
        survivor_accel += (-1, 0)
    a = np.sqrt(np.sum(survivor_accel**2))
    v = np.linalg.norm(survivor.vbar)
    if a != 0:
        survivor.abar = survivor_accel/a*survivor.maxAccel
    elif v != 0:
        survivor.vbar = np.zeros(2)
    #handle events
    for event in pygame.event.get():
        if event.type == QUIT:
            pygame.quit()
            sys.exit()
        elif event.type == pygame.MOUSEBUTTONDOWN:
            # Set the x, y postions of the mouse click
            x, y = event.pos
            for l in lampGroup:
                if l.rect.collidepoint(x, y):                  
                    l.toggle()
    
    #20 random zombies
    if random.randint(0, 1000) > 100:
        randZs = pygame.sprite.Group()
        zIndexes = [random.randint(0, len(zombieGroup)) for i in range(20)]
        i = 0
        for z in zombieGroup:
            if i in zIndexes:
                randZs.add(z)
                zIndexes.remove(i)
            i += 1
    dt = 0
    if clock.get_fps() != 0:
        dt = 1/clock.get_fps()
    zombieGroup.update(dt, survivor, lampGroup, randZs, \
        level1.interactiveObjects)
    gui1.update()
    lampGroup.update()
    level1.update()
    survivor.update(dt, zombieGroup, gui1, level1.interactiveObjects, level1)
    draw()
    
    clock.tick(fps_target)
pygame.quit()