import pygame
import numpy as np
import random
import level

random.seed()

'''
def get_force(self, xbar):
    dist_bar = self.xbar - xbar
    dist = np.linalg.norm(dist_bar)
    if dist != 0 and dist < self.detection_range:
        force = np.zeros(2)
        for strength,scaling in zip(self.strengths,self.scalings):
            force += dist_bar/dist*strength/(dist**scaling)
        return force
    else:
        return np.zeros(2)'''
  
def scale_rotate_image(image, scale, angle):
    return pygame.transform.rotozoom(image, angle, scale)
    
def color_surface(surface, red, green, blue):
    arr = pygame.surfarray.pixels3d(surface)
    arr[:,:,0] += red
    arr[:,:,1] += green
    arr[:,:,2] += blue
        
class Zombie(pygame.sprite.Sprite):
    def __init__(self, group, xbar): 
        pygame.sprite.Sprite.__init__(self, group)
        
        image = pygame.image.load('zombie.png')
        image.convert_alpha()
        rnd = random.randint(0,10) #power rating for zombie
        
        self.maxVel = 50.+2*rnd
        self.maxAccel = self.maxVel/0.5
        
        color_surface(image, rnd*5, rnd*-4, rnd*2)
        self.image = pygame.transform.rotozoom(image, random.randint(-20,20), .7 + .06*rnd)
        
        self.xbar = np.array(xbar)
        self.rect = self.image.get_rect()
        self.rect.center = xbar
        self.vbar = np.zeros(2)
        
        self.radius = self.image.get_rect().width/2
        
        self.currentBehavior = 'wander'
        self.survivorDetectRange = 400
        self.survivorLostRange = 500
        
        self.strengths = {Zombie:[-1e9, 1e7], Survivor:[3e8], 'OnLamp': [-3e12]}
        self.scalings = {Zombie:[2, 1], Survivor:[1], 'OnLamp': [2.5]}
        
    def onCollide(self, s):
        pass
        
    def update(self, dt, survivor, lamps, zombies, interactives):
        #update position and velocity
        self.xbar += self.vbar*dt
        self.rect.center = self.xbar
        #state machine transitions
        rbar_toSurvivor = survivor.xbar - self.xbar
        r_toSurvivor = np.linalg.norm(rbar_toSurvivor)
        if self.currentBehavior == 'wander':
            #check if survivor is within detection range
            if r_toSurvivor < self.survivorDetectRange:
                self.currentBehavior = 'chase'
                self.maxVel *= 2
        elif self.currentBehavior == 'chase':
            if r_toSurvivor > self.survivorLostRange:
                self.currentBehavior = 'wander'
                self.maxVel /= 2
        #state machine behaviors   
        abar = np.zeros(2)    
        for g in [zombies, [survivor]]:
                for s in g:
                    if s == self:
                        continue
                    for strength, scaling in zip( \
                            self.strengths[s.__class__], \
                            self.scalings[s.__class__]):
                        if s.__class__ == Zombie:
                            strength /= len(zombies)
                        abar += self.getAccelFrom(s.xbar, strength, scaling)
        for l in lamps:
            if l.isOn():
                for strength, scaling in zip( \
                            self.strengths['OnLamp'], \
                            self.scalings['OnLamp']):
                    abar += self.getAccelFrom(l.xbar, strength, scaling)
        
        for i in interactives:
                if i.__class__ == level.BouncyWall and pygame.sprite.collide_rect(self, i):
                    i.bounce(self)
                if i.__class__ == level.BouncyCircle and pygame.sprite.collide_circle(self, i):
                    i.bounce(self)
        
        #check that new velocity and acceleration are within bounds
        a = np.sqrt(np.sum(abar**2))
        if a > self.maxAccel:
            abar *= self.maxAccel/a
        self.vbar += abar*dt
        v = np.sqrt(np.sum(self.vbar**2))
        if v > self.maxVel:
            self.vbar *= self.maxVel/v
    
    def getAccelFrom(self, xbar, strength, scaling):
        rbar = xbar - self.xbar
        r = np.linalg.norm(rbar)
        if r != 0:
            rhat = rbar/r
            return strength*rhat/r**scaling
        else:
            return 0
        
            
class Lamp(pygame.sprite.Sprite):
    _imageOff = pygame.image.load('lamp_off.png')
    _imageOn = pygame.image.load('lamp_on.png')
    
    def __init__(self, group, xbar, startOn): 
        pygame.sprite.Sprite.__init__(self, group)
        self.xbar = np.array(xbar)
        self.rect = Lamp._imageOff.get_rect()
        self.rect.center = self.xbar
        self.rect.top = self.rect.top + self.rect.height/2
        self.imageOff = Lamp._imageOff
        self.imageOn = Lamp._imageOn
        if startOn:
            self.setLampOn()
        else:            
            self.setLampOff()

    def toggle(self):        
        if self.isOn():
            self.setLampOff()
        else:
            self.setLampOn()

    def update(self):
        if random.randint(0, 1000) > 990:
            self.toggle()
            
    def isOn(self):
        return self.image == self._imageOn

    def setLampOn(self):
        self.image = self.imageOn
        
    def setLampOff(self):
        self.image = self.imageOff
        
class Survivor(pygame.sprite.Sprite):
    maxVel = 150.
    maxAccel = 200.
    image = pygame.image.load('ape.png')
    damping = 0.
    
    def __init__(self, group, xbar):
        pygame.sprite.Sprite.__init__(self, group)
        self.xbar = np.array(xbar)
        self.image = pygame.Surface((95, 94))
        self.rect = self.image.get_rect()
        self.rect.center = xbar
        self.vbar = np.zeros(2)
        self.abar = np.zeros(2)
        self.coverEyes = False
        self.curr_frame_num = 0
        self.curr_patch_num = 0
        self.radius = self.image.get_rect().height/2
        self.isInvicible = False
        
    def update(self, dt, zombies, gui, interactives, lvl) :       
        self.curr_frame_num = (self.curr_frame_num+1) % 10
        update = False
        if not self.coverEyes and self.curr_frame_num == 0:
            self.curr_patch_num = 0
            update = True
        elif not self.coverEyes and self.curr_frame_num == 5:
            self.curr_patch_num = 1
            update = True
        elif self.coverEyes:
            self.curr_patch_num = 2
            update = True
        if update:
            patch_rect = (self.curr_patch_num * 95, 0,
                           95, self.image.get_height())
            self.image = Survivor.image.subsurface(patch_rect)
            
        if gui.alive:
            for z in zombies:
                if self.isInvicible:
                    break
                if pygame.sprite.collide_circle(self, z):
                    self.isInvicible = True
                    self.coverEyes = True
                    gui.gotBit()
                    self.invCounter = 2.
            if self.isInvicible:
                self.invCounter -= dt
                if self.invCounter <= 0:
                    self.isInvicible = False
                    self.coverEyes = False
            for i in interactives:
                if i.__class__ == level.BouncyWall and pygame.sprite.collide_rect(self, i):
                    i.bounce(self)
                if i.__class__ == level.BouncyCircle and pygame.sprite.collide_circle(self, i):
                    i.bounce(self)
            if (max(lvl.originLocation) > (lvl.size/2 - 2*self.radius) or \
                    min(lvl.originLocation) < (-lvl.size/2 + 2*self.radius) ) \
                    and not self.isInvicible:
                self.isInvicible = True
                self.coverEyes = True
                gui.gotBit()
                self.invCounter = 2.

            #update position and velocity
            self.xbar += self.vbar*dt
            self.rect.center = self.xbar
            
            self.vbar += self.abar*dt
            v = np.sqrt(np.sum(self.vbar**2))
            if v > Survivor.maxVel:
                self.vbar *= Survivor.maxVel/v
  